/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCSSRule.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/PrivateGdomCSSRule.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateIntGdomCSSRule.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMCSSRule::GDOMCSSRule(PassRefPtr<WebCore::CSSRule> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMCSSRule::~GDOMCSSRule()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CSSRule.h"
#include <gdom/GdomCSSRule.h>
#include <gdom/PrivateGdomCSSRule.h>
#include <gdom/PrivateIntGdomCSSRule.h>
#include <gdom/GdomCSSRuleprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCSSRule, gdom_css_rule, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomCSSRulePrivate {
    class WebCore::CSSRule *item;
};

#define GDOM_CSS_RULE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CSS_RULE, GdomCSSRulePrivate))
*/
G_END_DECLS
WebCore::CSSRule *
gdom_css_rule_get_private(GdomCSSRule *request)
{
    GDOMCSSRule *res;
    res = static_cast<GDOMCSSRule*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CSSRule*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_TYPE,
    PROP_CSS_TEXT,
    PROP_PARENT_STYLE_SHEET,
    PROP_PARENT_RULE,
};


static void gdom_css_rule_finalize(GObject* object)
{
    /*GdomCSSRule* request = GDOM_CSS_RULE(object);*/
    /*GdomCSSRulePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_css_rule_parent_class)->finalize(object);
}

static void gdom_css_rule__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSRule *self = GDOM_CSS_RULE(object);
    WebCore::CSSRule * gself = (gdom_css_rule_get_private(self));
    switch (prop_id) {
     case PROP_CSS_TEXT:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setCssText(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_rule__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSRule *self = GDOM_CSS_RULE(object);
    WebCore::CSSRule * gself = (gdom_css_rule_get_private(self));
    switch (prop_id) {
    case PROP_TYPE:
    {
         g_value_set_uint (value, gself->type());
         break;
    }
    case PROP_CSS_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->cssText()));
         break;
    }
    case PROP_PARENT_STYLE_SHEET:
    {
         PassRefPtr<WebCore::CSSStyleSheet> ptr = gself->parentStyleSheet();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PARENT_RULE:
    {
         PassRefPtr<WebCore::CSSRule> ptr = gself->parentRule();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_rule_class_init(GdomCSSRuleClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_css_rule_finalize;
    g_klass->set_property = gdom_css_rule__set_property_fn;
    g_klass->get_property = gdom_css_rule__get_property_fn;

    g_object_class_install_property (g_klass, PROP_TYPE,
            g_param_spec_uint ("type", /* name */
                "css_rule_type", /* short description */
                "read-only  gushort CSSRule.type", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CSS_TEXT,
            g_param_spec_string ("css_text", /* name */
                "css_rule_css_text", /* short description */
                "read-write  gchar * CSSRule.css_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PARENT_STYLE_SHEET,
            g_param_spec_object ("parent_style_sheet", /* name */
                "css_rule_parent_style_sheet", /* short description */
                "read-only  GdomCSSStyleSheet * CSSRule.parent_style_sheet", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_STYLE_SHEET, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PARENT_RULE,
            g_param_spec_object ("parent_rule", /* name */
                "css_rule_parent_rule", /* short description */
                "read-only  GdomCSSRule * CSSRule.parent_rule", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_RULE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCSSRulePrivate));*/
}

static void gdom_css_rule_init(GdomCSSRule* request)
{
    /*GdomCSSRulePrivate* priv = GDOM_CSS_RULE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

